body {
    font-family: Ubuntu, 'Bitstream Vera Sans', 'DejaVu Sans', Tahoma, sans-serif;
    font-size: 13px;
    line-height: 18px;
    color: #333;
}
h1, h2, h3, h4, h5, h6 {
    font-weight: normal;
}
h1 {
    font-size: 30px;
    line-height: 34px;
}
h2 {
    margin-top: 16px;
    margin-bottom: 8px;
    font-size: 22px;
    line-height: 26px;
}
h2.super-size {
    font-size: 120px;
    line-height: 100px;
}
h3 {
    margin-top: 16px;
    margin-bottom: 8px;
    font-size: 16px;
    line-height: 20px;
}
p {
    width: auto;
    margin-bottom: 0.8em;
}
p.large {
    font-size: 24px;
}
p.medium {
    font-size: 16px;
}
p.secondary {
    color: #AEA79F;
}
pre, code, samp, tt, .console {
    font-family: 'Ubuntu Mono', monospace;
    margin-bottom: 0.8em;
}
a {
    color: #dd4814;
    text-decoration: none;
}
a:hover, a:active {
    text-decoration: underline;
}
img, a img {
    border: none;
}
em {
    font-style: italic;
}
strong {
    font-weight: bold;
}
