.data-list {
    list-style: none;
    margin: 0;
    padding: 0;
}
.data-list h4 {
    margin-bottom: 5px;
}
.data-list span {
    font-size: 16px;
}
.data-list .block {
    word-wrap: break-word;
}
