.block {
    border: 0 solid #e5e2e0;
    margin: 0 10px 10px 10px;
}
.block.first {
    clear: left;
    margin-left: 0;
}
.block.last {
    clear: right;
    margin-right: 0;
}
.block.full-width {
    width: 100%;
    margin-left: 0;
    margin-right: 0;
    padding-bottom: 10px;
    border-bottom-width: 1px;
}
.block.auto-width {
    display: inline-block;
}
.size1,
.size2,
.size3,
.size4,
.size5,
.size6,
.size7,
.size8,
.size9,
.size10,
.size11,
.size12 {
    float: left;
}
.size1 {
    width: 60px;
}
.size2 {
    width: 120px;
}
.size3 {
    width: 180px;
}
.size4 {
    width: 240px;
}
.size5 {
    width: 300px;
}
.size6 {
    width: 360px;
}
.size7 {
    width: 420px;
}
.size8 {
    width: 480px;
}
.size9 {
    width: 540px;
}
.size10 {
    width: 600px;
}
.size11 {
    width: 660px;
}
.size12 {
    width: 720px;
}
